#import "DrawingView.h"
#import "LineSmoothing.h"

@interface DrawingView ()

@property (strong, nonatomic, readonly) NSMutableArray* lines;

@end

@implementation DrawingView

@synthesize lines = _lines;
- (NSMutableArray*)lines
{
    if (_lines == nil) {
        _lines = [[NSMutableArray alloc] init];
    }
    return _lines;
}

- (void)moveToPoint:(CGPoint)point
{
    NSMutableArray* line = [[NSMutableArray alloc] init];
    [line addObject:[NSValue valueWithCGPoint:point]];
    [self.lines addObject:line];
    [self setNeedsDisplay];
}

- (void)addLineToPoint:(CGPoint)point
{
    NSMutableArray* line = [self.lines lastObject];
    [line addObject:[NSValue valueWithCGPoint:point]];
    [self setNeedsDisplay];
}

// UIKit drawing
- (void)drawRect:(CGRect)rect
{
    [[UIColor whiteColor] setFill];
    UIRectFill(self.bounds);

    UIBezierPath* path = [UIBezierPath bezierPath];
    path.lineCapStyle = kCGLineCapRound;
    path.lineWidth = 5.;
    [[UIColor redColor] setStroke];

    for (NSUInteger i = 0; i < [self.lines count]; i++) {
        NSArray* line = self.lines[i];
        if ([line count] == 0) {
        } else {
            CGPoint point = [line[0] CGPointValue];
            [path moveToPoint:point];
            if ([line count] == 1) {
                [path addLineToPoint:point];
            } else {
                for (NSUInteger j = 1; j < [line count]; j++) {
                    CGPoint point2 = [line[j] CGPointValue];
                    [path addLineToPoint:point2];
                }
            }
            [path stroke];
        }
    }
}

// CoreGraphics drawing
//- (void)drawRect:(CGRect)rect
//{
//    CGContextRef ctx = UIGraphicsGetCurrentContext();
//
//    CGContextSaveGState(ctx);
//
//    CGContextSetLineCap(ctx, kCGLineCapRound);
//
//    CGContextSetFillColorWithColor(ctx, [UIColor whiteColor].CGColor);
//    CGContextFillRect(ctx, self.bounds);
//
//    CGContextSetLineWidth(ctx, 5.);
//    CGContextSetRGBStrokeColor(ctx, 1., 0., 0., 1.);
//
//    for (NSUInteger i = 0; i < [self.lines count]; i++) {
//        NSArray* line = self.lines[i];
//        if ([line count] == 0) {
//        } else {
//            CGPoint point = [line[0] CGPointValue];
//            CGContextMoveToPoint(ctx, point.x, point.y);
//            if ([line count] == 1) {
//                CGContextAddLineToPoint(ctx, point.x, point.y);
//            } else {
//                for (NSUInteger j = 1; j < [line count]; j++) {
//                    CGPoint point2 = [line[j] CGPointValue];
//                    CGContextAddLineToPoint(ctx, point2.x, point2.y);
//                }
//            }
//            CGContextStrokePath(ctx);
//        }
//    }
//
//    CGContextRestoreGState(ctx);
//}

// UIKit smooth drawing
//- (void)drawRect:(CGRect)rect
//{
//    [[UIColor whiteColor] setFill];
//    UIRectFill(self.bounds);
//
//    UIBezierPath* path = [UIBezierPath bezierPath];
//    path.lineCapStyle = kCGLineCapRound;
//    path.lineWidth = 5.;
//    [[UIColor redColor] setStroke];
//
//    for (NSUInteger i = 0; i < [self.lines count]; i++) {
//        NSArray* line = self.lines[i];
//        if ([line count] == 0) {
//        } else {
//            CGPoint point = [line[0] CGPointValue];
//            [path moveToPoint:point];
//            if ([line count] == 1) {
//                [path addLineToPoint:point];
//            } else if ([line count] == 2) {
//                CGPoint endPoint = [line[1] CGPointValue];
//                [path addLineToPoint:endPoint];
//            } else {
//                NSArray* controlPoints = [LineSmoothing cubicBezierCurveControlPointsForLine:line];
//                for (NSUInteger j = 1; j < [line count]; j++) {
//                    CGPoint fcp = [controlPoints[0][j - 1] CGPointValue];
//                    CGPoint scp = [controlPoints[1][j - 1] CGPointValue];
//                    CGPoint endPoint = [line[j] CGPointValue];
//                    [path addCurveToPoint:endPoint
//                            controlPoint1:fcp
//                            controlPoint2:scp];
//                }
//            }
//            [path stroke];
//        }
//    }
//}

// CoreGraphics smooth drawing
//- (void)drawRect:(CGRect)rect
//{
//    CGContextRef ctx = UIGraphicsGetCurrentContext();
//
//    CGContextSaveGState(ctx);
//
//    CGContextSetLineCap(ctx, kCGLineCapRound);
//
//    CGContextSetFillColorWithColor(ctx, [UIColor whiteColor].CGColor);
//    CGContextFillRect(ctx, self.bounds);
//
//    CGContextSetLineWidth(ctx, 5.);
//    CGContextSetRGBStrokeColor(ctx, 1., 0., 0., 1.);
//
//    for (NSUInteger i = 0; i < [self.lines count]; i++) {
//        NSArray* line = self.lines[i];
//        if ([line count] == 0) {
//        } else {
//            CGPoint point = [line[0] CGPointValue];
//            CGContextMoveToPoint(ctx, point.x, point.y);
//            if ([line count] == 1) {
//                CGContextAddLineToPoint(ctx, point.x, point.y);
//            } else if ([line count] == 2) {
//                CGPoint endPoint = [line[1] CGPointValue];
//                CGContextAddLineToPoint(ctx, endPoint.x, endPoint.y);
//            } else {
//                NSArray* controlPoints = [LineSmoothing cubicBezierCurveControlPointsForLine:line];
//                for (NSUInteger j = 1; j < [line count]; j++) {
//                    CGPoint fcp = [controlPoints[0][j - 1] CGPointValue];
//                    CGPoint scp = [controlPoints[1][j - 1] CGPointValue];
//                    CGPoint endPoint = [line[j] CGPointValue];
//                    CGContextAddCurveToPoint(ctx, fcp.x, fcp.y, scp.x, scp.y, endPoint.x, endPoint.y);
//                }
//            }
//            CGContextStrokePath(ctx);
//        }
//    }
//
//    CGContextRestoreGState(ctx);
//}

@end
