#import "LineSmoothing.h"

@implementation LineSmoothing

+ (NSArray*)cubicBezierCurveControlPointsForLine:(NSArray*)line
{
    NSUInteger length = [line count];

    CGPoint points[length];
    for (NSUInteger i = 0; i < length; i++) {
        points[i] = [line[i] CGPointValue];
    }

    length--;

    CGPoint firstControlPoints[length];
    CGPoint secondControlPoints[length];

    double rhs[length];
    for (int i = 1; i < length - 1; i++) {
        rhs[i] = 4 * points[i].x + 2 * points[i + 1].x;
    }
    rhs[0] = points[0].x + 2 * points[1].x;
    rhs[length - 1] = (8 * points[length - 1].x + points[length].x) / 2.0;
    double x[length];
    double tmp[length];
    double b = 2.0;
    x[0] = rhs[0] / b;
    for (int i = 1; i < length; i++) {
        tmp[i] = 1 / b;
        b = (i < length - 1 ? 4.0 : 3.5) - tmp[i];
        x[i] = (rhs[i] - x[i - 1]) / b;
    }
    for (int i = 1; i < length; i++) {
        x[length - i - 1] -= tmp[length - i] * x[length - i];
    }
    for (int i = 1; i < length - 1; ++i) {
        rhs[i] = 4 * points[i].y + 2 * points[i + 1].y;
    }
    rhs[0] = points[0].y + 2 * points[1].y;
    rhs[length - 1] = (8 * points[length - 1].y + points[length].y) / 2.0;
    double y[length];
    double tmp2[length];
    b = 2.0;
    y[0] = rhs[0] / b;
    for (int i = 1; i < length; i++) {
        tmp2[i] = 1 / b;
        b = (i < length - 1 ? 4.0 : 3.5) - tmp2[i];
        y[i] = (rhs[i] - y[i - 1]) / b;
    }
    for (int i = 1; i < length; i++) {
        y[length - i - 1] -= tmp2[length - i] * y[length - i];
    }
    for (int i = 0; i < length; ++i) {
        firstControlPoints[i] = CGPointMake(x[i], y[i]);
        if (i < length - 1) {
            secondControlPoints[i] = CGPointMake(2 * points[i + 1].x - x[i + 1], 2 * points[i + 1].y - y[i + 1]);
        } else {
            secondControlPoints[i] = CGPointMake((points[length].x + x[length - 1]) / 2, (points[length].y + y[length - 1]) / 2);
        }
    }

    NSMutableArray* fcp = [[NSMutableArray alloc] init];
    for (int i = 0; i < length; i++) {
        [fcp addObject:[NSValue valueWithCGPoint:firstControlPoints[i]]];
    }

    NSMutableArray* scp = [[NSMutableArray alloc] init];
    for (int i = 0; i < length; i++) {
        [scp addObject:[NSValue valueWithCGPoint:secondControlPoints[i]]];
    }

    return @[
        [fcp copy],
        [scp copy]
    ];
}

@end
