#import "MyGestureRecognizer.h"
#import <UIKit/UIGestureRecognizerSubclass.h>

@interface MyGestureRecognizer ()

@property (nonatomic) CGPoint startingPoint;
@property (strong, nonatomic) UITouch* touch;

@end

@implementation MyGestureRecognizer

- (void)reset
{
    self.touch = nil;
    self.state = UIGestureRecognizerStatePossible;
}

- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event
{
    if (self.touch == nil) {
        self.touch = [touches anyObject];
        self.startingPoint = [self.touch locationInView:self.view];
        self.state = UIGestureRecognizerStateBegan;
    }
}

- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event
{
    if ([touches containsObject:self.touch]) {
        self.state = UIGestureRecognizerStateChanged;
    }
}

- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event
{
    if ([touches containsObject:self.touch]) {
        self.state = UIGestureRecognizerStateEnded;
    }
}

- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event
{
    if ([touches containsObject:self.touch]) {
        self.state = UIGestureRecognizerStateCancelled;
    }
}

- (CGPoint)translationInView:(UIView*)view
{
    CGPoint currentPoint = [self.touch locationInView:self.view];
    CGPoint convertedStartingPoint = [view convertPoint:self.startingPoint
                                               fromView:self.view];
    CGPoint convertedTranslation = CGPointMake(convertedStartingPoint.x + currentPoint.x - self.startingPoint.x, convertedStartingPoint.y + currentPoint.y - self.startingPoint.y);
    return convertedTranslation;
}

@end
